function varargout = TrackFilter_GUI(varargin)
% TRACKFILTER_GUI MATLAB code for TrackFilter_GUI.fig
%      TRACKFILTER_GUI, by itself, creates a new TRACKFILTER_GUI or raises the existing
%      singleton*.
%
%      H = TRACKFILTER_GUI returns the handle to a new TRACKFILTER_GUI or the handle to
%      the existing singleton*.
%
%      TRACKFILTER_GUI('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in TRACKFILTER_GUI.M with the given input arguments.
%
%      TRACKFILTER_GUI('Property','Value',...) creates a new TRACKFILTER_GUI or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before TrackFilter_GUI_OpeningFcn gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to TrackFilter_GUI_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help TrackFilter_GUI

% Last Modified by GUIDE v2.5 07-Jul-2015 12:28:14

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @TrackFilter_GUI_OpeningFcn, ...
    'gui_OutputFcn',  @TrackFilter_GUI_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT


% --- Executes just before TrackFilter_GUI is made visible.
function TrackFilter_GUI_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to TrackFilter_GUI (see VARARGIN)

% Choose default command line output for TrackFilter_GUI
handles.output = hObject;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set initial simulation parameter values, index vectors, etc.


% Define ground truth target position and velocity
N1 = 30;  % number of time steps simulated at v1
Nt = 10;  % number of time steps where acceleration is present
N2 = 30;  % number of time steps simulated at v2
Ts = 1;  % time step in seconds
t = (0:N1+Nt+N2-1)'*Ts;  % time vector
t_lim = [min(t),max(t)];

x_gt = zeros(size(t));
v_gt = zeros(size(t));

% actual target parameters
v1 = 50; % 50 m/s nominal for first part of simulation
v2 = 0;  % stationary for second part of simulation
x0 = 1000;  % starts at origin

% Set up true (ground truth) velocity vector
A = (v2 - v1)/(Nt-1);  % acceleration during the maneuver
v_gt(1:N1) = v1;
v_gt(N1+1:N1+Nt) = v1 + A*(0:Nt-1);
v_gt(N1+Nt+1:N1+Nt+N2) = v2;

v_ylim = [min(v1,v2)-max(v_gt)/2,max(v1,v2)+max(v_gt)/2];  % used later for plotting niceness

% compute true (ground truth) position
x_gt(1) = x0;
for n = 2:N1+Nt+N2
    x_gt(n) = x_gt(n-1) + v_gt(n-1)*Ts;
end

% Set initial process and measurement variances.
MeasStdDev0 = 100; % initial measurement noise std dev
ProcessStdDev0 = 2*abs(A);  % initial acceleration noise std dev

% Set slider limits, where they need to be different from the defaults
% The default range is [0,1] with an initial value of 0.  That works fine
% for alpha and beta, and the min of 0 works for the measurement and
% process noise, but the maximums need to be set.  Also set SliderStep for these
% to be 1% and 10% of the range.
set(handles.MeasStdDev_slider,'Max',0.2*max(x_gt)); % limit to 20% of max position
set(handles.ProcessStdDev_slider,'Max',0.5*max(v_gt)); % limit to 50% of max velocity

% Compute initial tracking index and, from that, initial steady-state alpha
% and beta
TI0 = Ts^2*sqrt(ProcessStdDev0)/sqrt(MeasStdDev0);
alpha0 =  -TI0^2/8 - TI0 + (4+TI0)*sqrt(TI0^2 + 8*TI0)/8;
beta0 = TI0^2/4 + TI0 - TI0*sqrt(TI0^2 + 8*TI0)/4;

% Initialize GUI elements
MeasStdDev_string = num2str(MeasStdDev0);
set(handles.MeasStdDev_edit,'String',MeasStdDev_string);
set(handles.MeasStdDev_slider,'Value',MeasStdDev0);

ProcessStdDev_string = num2str(ProcessStdDev0);
set(handles.ProcessStdDev_edit,'String',ProcessStdDev_string);
set(handles.ProcessStdDev_slider,'Value',ProcessStdDev0);

TI_string = num2str(TI0);
set(handles.TrkIndex_static,'String',['Tracking Index = ',TI_string]);

alpha_string = num2str(alpha0);
set(handles.alpha_edit,'String',alpha_string);
set(handles.alpha_slider,'Value',alpha0);

beta_string = num2str(beta0);
set(handles.beta_edit,'String',beta_string);
set(handles.beta_slider,'Value',beta0);

% create additional handles data objects for various signals and variables
% defined in this GUI figure file, and update handles
handles.x_gt = x_gt;  % position ground truth
handles.v_gt = v_gt;  % velocity ground truth
handles.Ts = Ts;  % time step
handles.t = t;  % time vector
handles.t_lim = t_lim;  % time limits (for plot axis control)
handles.xn = [];  % noisy position measurements
handles.xs = [];  % smoothed position (alpha-beta)
handles.vs = [];  % smoothed velocity (alpha-beta)
handles.xs_line_on = false;  % denotes whether or not there is a currently plotted xs line
handles.vs_line_on = false;  % denotes whether or not there is a currently plotted vs line
handles.xs_line = [];  % identifier for currently plotted xs line
handles.vs_line = [];  % identifier for currently plotted vs line
handles.xsK = [];  % smoothed position (Kalman)
handles.vsK = [];  % smoothed velocity (Kalman)
handles.xsK_line_on = false;  % denotes whether or not there is a currently plotted xsK line
handles.vsK_line_on = false;  % denotes whether or not there is a currently plotted vsK line
handles.xsK_line = [];  % identifier for currently plotted xsK line
handles.vsK_line = [];  % identifier for currently plotted vsK line
handles.v_ylim = v_ylim;  % velocity limits (for velocity plot axis control)
handles.MeasStdDev = MeasStdDev0;  % measurement (position) noise variance
handles.ProcessStdDev = ProcessStdDev0;  % process (acceleration) noise variance
handles.TI = TI0;  % initial value of tracking index
handles.alpha = alpha0;  % initial value of alpha
handles.beta = beta0;  % initial value of beta
handles.ss_alpha = alpha0;  % initialize steady state alpha
handles.ss_beta = beta0; % initialize steady state beta
handles.run_alphabeta = 0;  % alpha beta filter is initially off
handles.run_Kalman = 0;  % Kalman filter is initially off
handles.ss_alphabeta = 0;  % use of steady state alpha-beta values is initially off

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes TrackFilter_GUI wait for user response (see UIRESUME)
% uiwait(handles.figure1);


% --- Outputs from this function are returned to the command line.
function varargout = TrackFilter_GUI_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in NewMeasurements_pushbutton.
function NewMeasurements_pushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to NewMeasurements_pushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Create new measurement data and refresh the plots
GenerateNewData(hObject,handles)
PlotNewData(hObject,handles)



% --- Executes on button press in RunFilters_pushbutton.
function RunFilters_pushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to RunFilters_pushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
RunFilters(hObject,handles)
PlotFilters(hObject,handles)


% --- Executes on slider movement.
function MeasStdDev_slider_Callback(hObject, eventdata, handles)
% hObject    handle to MeasStdDev_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider

% Update the edit box to reflect the new slider position value
MeasStdDev = get(handles.MeasStdDev_slider,'Value');
MeasStdDev_string = num2str(MeasStdDev);
set(handles.MeasStdDev_edit,'String',MeasStdDev_string);

% Update handles structure
guidata(hObject, handles);

UpdateTI(hObject,handles)


% --- Executes during object creation, after setting all properties.
function MeasStdDev_slider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to MeasStdDev_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


function MeasStdDev_edit_Callback(hObject, eventdata, handles)
% hObject    handle to MeasStdDev_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of MeasStdDev_edit as text
%        str2double(get(hObject,'String')) returns contents of MeasStdDev_edit as a double

% Update the slider position to match the data entered in the edit box
MeasStdDev_string = get(handles.MeasStdDev_edit,'String');
MeasStdDev = str2double(MeasStdDev_string);
MeasStdDev_max = get(handles.MeasStdDev_slider,'Max');
MeasStdDev = min(MeasStdDev,MeasStdDev_max);
MeasStdDev_string = num2str(MeasStdDev);
set(handles.MeasStdDev_edit,'String',MeasStdDev_string);
set(handles.MeasStdDev_slider,'Value',MeasStdDev);

% Update handles structure
guidata(hObject, handles);

UpdateTI(hObject,handles)


% --- Executes during object creation, after setting all properties.
function MeasStdDev_edit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to MeasStdDev_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on slider movement.
function ProcessStdDev_slider_Callback(hObject, eventdata, handles)
% hObject    handle to ProcessStdDev_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider

% Update the edit box to reflect the new slider position value
ProcessStdDev = get(handles.ProcessStdDev_slider,'Value');
ProcessStdDev_string = num2str(ProcessStdDev);
set(handles.ProcessStdDev_edit,'String',ProcessStdDev_string);

% Update handles structure
guidata(hObject, handles);

UpdateTI(hObject,handles)


% --- Executes during object creation, after setting all properties.
function ProcessStdDev_slider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ProcessStdDev_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


function ProcessStdDev_edit_Callback(hObject, eventdata, handles)
% hObject    handle to ProcessStdDev_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of ProcessStdDev_edit as text
%        str2double(get(hObject,'String')) returns contents of ProcessStdDev_edit as a double

% Update the slider position to match the data entered in the edit box
ProcessStdDev_string = get(handles.ProcessStdDev_edit,'String');
ProcessStdDev = str2double(ProcessStdDev_string);
ProcessStdDev_max = get(handles.ProcessStdDev_slider,'Max');
ProcessStdDev = min(ProcessStdDev,ProcessStdDev_max);
ProcessStdDev_string = num2str(ProcessStdDev);
set(handles.ProcessStdDev_edit,'String',ProcessStdDev_string);
set(handles.ProcessStdDev_slider,'Value',ProcessStdDev);

% Update handles structure
guidata(hObject, handles);

UpdateTI(hObject,handles)


% --- Executes during object creation, after setting all properties.
function ProcessStdDev_edit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to ProcessStdDev_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function TrkIndex_edit_Callback(hObject, eventdata, handles)
% hObject    handle to TrkIndex_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of TrkIndex_edit as text
%        str2double(get(hObject,'String')) returns contents of TrkIndex_edit as a double


% --- Executes during object creation, after setting all properties.
function TrkIndex_edit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to TrkIndex_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on slider movement.
function alpha_slider_Callback(hObject, eventdata, handles)
% hObject    handle to alpha_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider

% Update the edit box to reflect the new slider position value
alpha = get(handles.alpha_slider,'Value');
alpha_string = num2str(alpha);
set(handles.alpha_edit,'String',alpha_string);
handles.alpha = alpha;

% Update handles structure
guidata(hObject, handles);

% Update alpha-beta filtered data plot immediately
RunFilters(hObject,handles)
PlotFilters(hObject,handles)


% --- Executes during object creation, after setting all properties.
function alpha_slider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to alpha_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


function alpha_edit_Callback(hObject, eventdata, handles)
% hObject    handle to alpha_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of alpha_edit as text
%        str2double(get(hObject,'String')) returns contents of alpha_edit as a double

% Update the slider position to match the data entered in the edit box
alpha_string = get(handles.alpha_edit,'String');
alpha = str2double(alpha_string);
alpha = min(1,max(alpha,0)); % limits alpha to between 0 and 1
alpha_string = num2str(alpha);
set(handles.alpha_edit,'String',alpha_string);
set(handles.alpha_slider,'Value',alpha);
handles.alpha = alpha;

% Update handles structure
guidata(hObject, handles);

% Update alpha-beta filtered data plot immediately
RunFilters(hObject,handles)
PlotFilters(hObject,handles)


% --- Executes during object creation, after setting all properties.
function alpha_edit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to alpha_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


% --- Executes on slider movement.
function beta_slider_Callback(hObject, eventdata, handles)
% hObject    handle to beta_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'Value') returns position of slider
%        get(hObject,'Min') and get(hObject,'Max') to determine range of slider

% Update the edit box to reflect the new slider position value
beta = get(handles.beta_slider,'Value');
beta_string = num2str(beta);
set(handles.beta_edit,'String',beta_string);
handles.beta = beta;

% Update handles structure
guidata(hObject, handles);

% Update alpha-beta filtered data plot immediately
RunFilters(hObject,handles)
PlotFilters(hObject,handles)


% --- Executes during object creation, after setting all properties.
function beta_slider_CreateFcn(hObject, eventdata, handles)
% hObject    handle to beta_slider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: slider controls usually have a light gray background.
if isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor',[.9 .9 .9]);
end


function beta_edit_Callback(hObject, eventdata, handles)
% hObject    handle to beta_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hints: get(hObject,'String') returns contents of beta_edit as text
%        str2double(get(hObject,'String')) returns contents of beta_edit as a double

% Update the slider position to match the data entered in the edit box
beta_string = get(handles.beta_edit,'String');
beta = str2double(beta_string);
beta = min(1,max(beta,0)); % limits alpha to between 0 and 1
beta_string = num2str(beta);
set(handles.beta_edit,'String',beta_string);
set(handles.beta_slider,'Value',beta);
handles.beta = beta;

% Update handles structure
guidata(hObject, handles);

% Update alpha-beta filtered data plot immediately
RunFilters(hObject,handles)
PlotFilters(hObject,handles)


% --- Executes during object creation, after setting all properties.
function beta_edit_CreateFcn(hObject, eventdata, handles)
% hObject    handle to beta_edit (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    empty - handles not created until after all CreateFcns called

% Hint: edit controls usually have a white background on Windows.
%       See ISPC and COMPUTER.
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end


function UpdateTI(hObject,handles)
% Function to update the current value of the tracking index shown in the
% GUI pane.  Also update the steady-state alpha and beta values.

% Retrieve needed variables
Ts = handles.Ts;
MeasStdDev_string = get(handles.MeasStdDev_edit,'String');
MeasStdDev = str2double(MeasStdDev_string);
ProcessStdDev_string = get(handles.ProcessStdDev_edit,'String');
ProcessStdDev = str2double(ProcessStdDev_string);

% Update the tracking index and update the figure data
TI = Ts^2*ProcessStdDev/MeasStdDev;
TI_string = num2str(TI);
set(handles.TrkIndex_static,'String',['Tracking Index = ',TI_string]);
handles.TI = TI;

handles.ss_alpha =  -TI^2/8 - TI + (4+TI)*sqrt(TI^2 + 8*TI)/8;
handles.ss_beta = TI^2/4 + TI - TI*sqrt(TI^2 + 8*TI)/4;

% Update handles structure
guidata(hObject, handles);


% --- Executes on button press in AlphaBeta_radiobutton.
function AlphaBeta_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to AlphaBeta_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of AlphaBeta_radiobutton

handles.run_alphabeta = get(hObject,'Value');

% Update handles structure
guidata(hObject, handles);


% --- Executes on button press in Kalman_radiobutton.
function Kalman_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to Kalman_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of Kalman_radiobutton

handles.run_Kalman = get(hObject,'Value');

% Update handles structure
guidata(hObject, handles);


% --- Executes on button press in SteadyStateAlphaBeta_radiobutton.
function SteadyStateAlphaBeta_radiobutton_Callback(hObject, eventdata, handles)
% hObject    handle to SteadyStateAlphaBeta_radiobutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Hint: get(hObject,'Value') returns toggle state of SteadyStateAlphaBeta_radiobutton

handles.ss_alphabeta = get(hObject,'Value');

% Update handles structure
guidata(hObject, handles);



function GenerateNewData(hObject,handles)
% Function to generate a set of noisy measurements

% Collect the variables needed
x_gt = handles.x_gt;

MeasStdDev_string = get(handles.MeasStdDev_edit,'String');
MeasStdDev = str2double(MeasStdDev_string);

% Form the noisymeasurements
xx = x_gt + MeasStdDev*randn(size(x_gt));

% Update the GUI data
handles.xn = xx;

% Update handles structure
guidata(hObject, handles);


function PlotNewData(hObject,handles)
% Function to refresh position and velocity graphs with new positiondata,
% and with position and velocity ground truth lines

handles = guidata(hObject);   % Don't know why I need to do this

% Retrieve needed variables
xn = handles.xn;  % noisy position data
x_gt = handles.x_gt; % position ground truth
v_gt = handles.v_gt;  % velocity ground truth
t = handles.t;  % time index
v_ylim = handles.v_ylim;
t_lim = handles.t_lim;

% Do the plots
axes(handles.Position_axes);
plot(t,x_gt,'LineWidth',2,'Color',[150 150 150]/255);  % fat grey line
xlim(t_lim)
hold on
plot(t,xn,'d');  % unconnected diamond markers
hold off
grid
xlabel('time')
ylabel('position (m)')

axes(handles.Velocity_axes);
plot(t,v_gt,'LineWidth',2,'Color',[150 150 150]/255);  % fat grey line
xlim(t_lim)
ylim(v_ylim)
grid
xlabel('time')
ylabel('velocity (m/s)')

% These are necessary so that the filter plotting routine doesn't generate
% an error
handles.xs_line = []; % will become reference to position graph line object
handles.vs_line = []; % will become reference to velocity graph line object
handles.xsK_line = []; % will become reference to position graph line object
handles.vsK_line = []; % will become reference to velocity graph line object

% Update handles structure
guidata(hObject, handles);


function RunFilters(hObject,handles)
% Function to call one or both filters as determined by the radio button
% selections

% check for existence of data to be filtered
if numel(handles.xn) == 0
    warndlg('You must create some new measurements before running the filters.',...
        'No Data!','modal')
    return
end

% Make sure at least one filter is selected to run
if (~handles.run_alphabeta) && (~handles.run_Kalman)
    warndlg('You must select at least one filter type to run.',...
        'No filter selected!','modal')
    return
end

% Run alpha-beta filter if selected
if (handles.run_alphabeta)
    AlphaBeta_filter(hObject,handles)
end

% Run Kalman filter if selected
if (handles.run_Kalman)
    Kalman_filter(hObject,handles)
end


function AlphaBeta_filter(hObject,handles)
% Function to apply the alpha-beta filter to the current data measurements

handles = guidata(hObject);   % Don't know why I need to do this

% First retrieve the needed variables
t = handles.t;
xn = handles.xn;
Ts = handles.Ts;
if (handles.ss_alphabeta)
    alpha = handles.ss_alpha;
    beta = handles.ss_beta;
    set(handles.alpha_slider,'Value',alpha);
    alpha_string = num2str(alpha);
    set(handles.alpha_edit,'String',alpha_string);
    handles.alpha = alpha;
    set(handles.beta_slider,'Value',beta);
    beta_string = num2str(beta);
    set(handles.beta_edit,'String',beta_string);
    handles.beta = beta;
else
    alpha = handles.alpha;
    beta = handles.beta;
end

% Now do the filtering.  Initialize using the first two data points.
vs = zeros(size(t));
vp = zeros(size(t));
xs = zeros(size(t));
xp = zeros(size(t));
e = zeros(size(t));

% Use first two samples to initialize velocity
xs(1) = xn(1);
xs(2) = xn(2);
vs(2) = (xn(2)-xn(1))/Ts;

for n = 3:length(t)  % prediction loop
    xp(n) = xs(n-1) + Ts*vs(n-1);
    vp(n) = vs(n-1);
    
    e(n) = xn(n) - xp(n);
    xs(n) = xp(n) + alpha*e(n);  % smoothed position
    vs(n) = vp(n) + beta*(e(n)/Ts);  % smoothed velocity
end

% Store the result and update handles
handles.xs = xs;
handles.vs = vs;

% Update handles structure
guidata(hObject, handles);


function Kalman_filter(hObject,handles)
% Function to apply the Kalmana filter to the current data measurements

handles = guidata(hObject);   % Don't know why I need to do this

% First retrieve the needed variables
t = handles.t;
xn = handles.xn;
Ts = handles.Ts;
ProcessStdDev = handles.ProcessStdDev;
MeasStdDev = handles.MeasStdDev;

% Now do the filtering.  Initialize using the first two data points.
xsK = zeros(size(t));
vsK = zeros(size(t));
MxK = zeros(2*length(t));
e = zeros(size(t));
k = zeros(2,length(t));

% Model
F = [1 Ts; 0 1];
% G = [1 0 ; 0 1 ];
G = [Ts^2/2  Ts]';
h = [1 0]';
I = [1 0; 0 1];
Sv = ProcessStdDev^2;  % process noise power

% % Initialize. "Big" value of M intended to discount initial state, since we
% % don't really know it anyway
% M = 1000*I;
% xxs = [x0 v1]';

% Alternate initialization per Bar-Shalom and Fortmann; see also FRSP book,
% 2nd ed., p. 423
xsK(1) = xn(1); xsK(2) = xn(2);
vsK(1) = 0; vsK(2) = (xn(2) - xn(1))/Ts;
xxs = [xsK(2) vsK(2)]';
nvar = MeasStdDev^2;
M = [nvar nvar/Ts; nvar/Ts 2*nvar/Ts^2];
k(1,:) = NaN;
k(2,:) = NaN;

for n = 3:length(t)  % prediction loop
    xxp = F*xxs;  % predicted position
    M = F*M*F' + G*Sv*G';  % MSE matrix update for prediction
    k(:,n) = M*h/(nvar + h'*M*h);  % Kalman gain update
    e(n) = (xn(n) - h'*xxp);  % innovation
    xxs = xxp + k(:,n)*e(n);  % corrected/smoothed position
    xsK(n) = xxs(1);  % smoothed poistion for later plotting
    vsK(n) = xxs(2);   % smoothed velocity for later plotting
    M = (I - k(:,n)*h')*M;  % MSE update for smoothing step
end

% Store the result and update handles
handles.xsK = xsK;
handles.vsK = vsK;

% Update handles structure
guidata(hObject, handles);


function PlotFilters(hObject,handles)
% Function to refresh position and velocity graphs with new positiondata,
% and with position and velocity ground truth lines

handles = guidata(hObject);   % Don't know why I need to do this

t = handles.t;  % time index

% Update the plots. We're adding to the ground truth and measurement plots, so
% have to use 'hold on'
axes(handles.Position_axes);
hold on
ylim('auto');
if (handles.xs_line_on)
    delete(handles.xs_line);
    handles.xs_line_on = false;
end
if (handles.run_alphabeta)
    handles.xs_line = plot(t,handles.xs,'r','LineWidth',1.5);  % fat red line
    handles.xs_line_on = true;
end

if (handles.xsK_line_on)
    delete(handles.xsK_line);
    handles.xsK_line_on = false;
end
if (handles.run_Kalman)
    handles.xsK_line = plot(t,handles.xsK,'Color',[0 0.5 0],'LineWidth',1.5);  % fat dark green line
    handles.xsK_line_on = true;
end
hold off

axes(handles.Velocity_axes);
hold on
ylim('auto');
if (handles.vs_line_on)
    delete(handles.vs_line);
    handles.vs_line_on = false;
end
if (handles.run_alphabeta)
    handles.vs_line = plot(t,handles.vs,'r','LineWidth',1.5);  % fat red  line
    handles.vs_line_on = true;
end

if (handles.vsK_line_on)
    delete(handles.vsK_line);
    handles.vsK_line_on = false;
end
if (handles.run_Kalman)
    handles.vsK_line = plot(t,handles.vsK,'Color',[0 0.5 0],'LineWidth',1.5);  % fat dark green  line
    handles.vsK_line_on = true;
end

hold off

% Update handles structure
guidata(hObject, handles);
