function T=threshold(Pfa,N)

% Takes matrix of Pfa values and a matrix of N 
% values and finds corresponding matrix of threshold values.
% Applicable to interference that is zero mean complex Gaussian
% noise with uncorrleated I and Q and unit variance.
%
% If Pfa and N are scalar, it finds one threshold T.
%
% If Pfa is a vector of values and N is a vector of
% the same length filled with the same value N, this
% gives you vector of thresholds for different Pfas 
% and fixed N.  Similarly, you can make the Pfa vector
% a constant and the N vector changing, and get thresholds
% for fixed Pfa and varying N.
%
% Finally, by using a matrix, you can get thresholds for a
% combination of different Pfas and Ns.
%
% A test case, verified one entry at a time using Dougherty's code:
%
% If Pfa = [1e-4 1e-6 1e-8] and N = [1 10 20]
%          [2e-4 2e-6 2e-8]         [5 15 25]
%
% (i.e., Pfa and N are the 2x3 matrics shown), then you should get
%
%    T = [ 9.2103   32.71   55.946]
%        [16.898    39.981  62.75 ]
%
% with the display format set to 'short G'.
%
% Mark A. Richards, June 2002.

% Rewritten to use MATLAB's gammaincinv function, and as a consequence
% greatly simplified, in Jun 2014.

if size(Pfa) ~= size(N)
    error('threshold: Error: Pfa and N not the same dimensions');
    return
end

T = gammaincinv(1-Pfa,N);