% Gnc_vs_N_and_Swerling_model
%
% Demonstration calling program for the "Detection Calculator" routines
%
% M file to compute noncoherent integration gain for Swerling 2 case and
% various values of N, Pd, and Pfa
%
%  Mark A. Richards
%  July 2014

clear all
close all

Pd = 0.9;
Pfa = 1e-3;

N = [2:10,12:2:20,25:5:50,60:10:100];

GncN0 = zeros(1,length(N));
GncN1 = zeros(1,length(N));
GncN2 = zeros(1,length(N));

% First do it for Swerling 0, 1, and 2 (nonfluc and exponential PDFs)

for n = 1:length(N)
    Nn = N(n);
    % SW0
    GncdB = SNRdB(Pd,Pfa,1,0) - SNRdB(Pd,Pfa,Nn,0);
    GncN0(n) = (10^(GncdB/10))/Nn;
    
    % SW1
    GncdB = SNRdB(Pd,Pfa,1,1) - SNRdB(Pd,Pfa,Nn,1);
    GncN1(n) = (10^(GncdB/10))/Nn;
    
    % SW2
    GncdB = SNRdB(Pd,Pfa,1,2) - SNRdB(Pd,Pfa,Nn,2);
    GncN2(n) = (10^(GncdB/10))/Nn;
    
end

figure
plot(N,[GncN0; GncN1; GncN2])
xlabel('N'); ylabel('Gnc/N')
grid
title(['Pd=',num2str(Pd),', Pfa=',num2str(Pfa)])
hline(1,'b')
legend('SW0','SW1','SW2','Location','Best')
shg

% Again for Swerling 0, 3, and 4 (nonfluc and chi-square PDFs)

GncN3 = zeros(1,length(N));
GncN4 = zeros(1,length(N));

for n = 1:length(N)
    Nn = N(n);
    % SW0
    GncdB = SNRdB(Pd,Pfa,1,0) - SNRdB(Pd,Pfa,Nn,0);
    GncN0(n) = (10^(GncdB/10))/Nn;
    
    % SW3
    GncdB = SNRdB(Pd,Pfa,1,3) - SNRdB(Pd,Pfa,Nn,3);
    GncN3(n) = (10^(GncdB/10))/Nn;
    
    % SW4
    GncdB = SNRdB(Pd,Pfa,1,4) - SNRdB(Pd,Pfa,Nn,4);
    GncN4(n) = (10^(GncdB/10))/Nn;
    
end

figure
plot(N,[GncN0; GncN3; GncN4])
xlabel('Pd'); ylabel('Gnc/N')
grid
title(['Pd=',num2str(Pd),', Pfa=',num2str(Pfa)])
hline(1,'b')
legend('SW0','SW3','SW4','Location','Best')
shg


