% Gnc_vs_Pd_and_N_for_SW2
%
% Demonstration calling program for the "Detection Calculator" routines
%
% Computes noncoherent integration gain vs. Pd and N for a Swerling 2 case.
%  This demosntrates the fact that the noncoherent integration gain can be
%  bigger than a factor of N for relatively high Pd in the SW2 case.
%
% Mark A. Richards
% July 2014

clear all
close all

N = [2,5,10,20,50,100];
Pd = 0.4:0.01:0.9;
Pfa = 1e-3;

iif2 = zeros(length(N),length(Pd));
for n = 1:length(N)
    for p = 1:length(Pd)
        Nn = N(n);
        Pdp = Pd(p);
        snr1 = gammaincinv(1-Pfa,1)/gammaincinv(1-Pdp,1) - 1;
        snrN = gammaincinv(1-Pfa,Nn)/gammaincinv(1-Pdp,Nn) - 1;
        Gnc = snr1/snrN;
        iif2(n,p) = Gnc/Nn;
    end
end

figure
plot(Pd,iif2)
xlabel('Pd'); ylabel('Gnc/N')
grid
title(['SW2, Pfa=',num2str(Pfa)])
hline(1,'b')
legend('N=2','5','10','20','50','100','Location','Best')
shg

% Repeat for SW4

Pfa = 1e-4;
Pd = 0.5:0.01:0.98;


iif4 = zeros(length(N),length(Pd));
for n = 1:length(N)
    for p = 1:length(Pd)
        Nn = N(n);
        Pdp = Pd(p);
        snr1 = 10^(SNRdB(Pdp,Pfa,1,4)/10);
        snrN = 10^(SNRdB(Pdp,Pfa,Nn,4)/10);
        Gnc = snr1/snrN;
        iif4(n,p) = Gnc/Nn;
    end
end

figure
plot(Pd,iif4)
xlabel('Pd'); ylabel('Gnc/N')
grid
title(['SW4, Pfa=',num2str(Pfa)])
hline(1,'b')
legend('N=2','5','10','20','50','100','Location','Best')
shg
