% Gnc_vs_Pd_and_Swerling_model
%
% Demonstration calling program for the "Detection Calculator" routines
%
% M file to compute noncoherent integration gain vs. Pd for various Swerilng models and
% specified  values of N and Pfa. Demosntrates that integration gain can be
% greater than N for SW2 or SW4, but not for SW0, 1, or 3.
%
% Mark A. Richards
% July 2014

clear all
close all

N = 10;
Pfa = 1e-3;

Pd = [0.4:0.02:0.9];

GncN0 = zeros(1,length(Pd));
GncN1 = zeros(1,length(Pd));
GncN2 = zeros(1,length(Pd));

% First do it for Swerling 0, 1, and 2 (nonfluc and exponential PDFs)

for p = 1:length(Pd)
    Pdp = Pd(p);
    % SW0
    GncdB = SNRdB(Pdp,Pfa,1,0) - SNRdB(Pdp,Pfa,N,0);
    GncN0(p) = (10^(GncdB/10))/N;
    
    % SW1
    GncdB = SNRdB(Pdp,Pfa,1,1) - SNRdB(Pdp,Pfa,N,1);
    GncN1(p) = (10^(GncdB/10))/N;
    
    % SW2
    GncdB = SNRdB(Pdp,Pfa,1,2) - SNRdB(Pdp,Pfa,N,2);
    GncN2(p) = (10^(GncdB/10))/N;
%     Gnc = 10^(SNRdB(Pdp,Pfa,1,2)/10) / 10^(SNRdB(Pdp,Pfa,N,2)/10);
%     GncN2(p) = Gnc/N;
    
end

figure
plot(Pd,[GncN0; GncN1; GncN2])
xlabel('Pd'); ylabel('Gnc/N')
grid
title(['N=',num2str(N),', Pfa=',num2str(Pfa)])
hline(1,'b')
legend('SW0','SW1','SW2','Location','Best')
shg

% Again for Swerling 0, 3, and 4 (nonfluc and chi-square PDFs)

GncN3 = zeros(1,length(Pd));
GncN4 = zeros(1,length(Pd));

for p = 1:length(Pd)
    Pdp = Pd(p);
    % SW0
    GncdB = SNRdB(Pdp,Pfa,1,0) - SNRdB(Pdp,Pfa,N,0);
    GncN0(p) = (10^(GncdB/10))/N;
    
    % SW3
    GncdB = SNRdB(Pdp,Pfa,1,3) - SNRdB(Pdp,Pfa,N,3);
    GncN3(p) = (10^(GncdB/10))/N;
    
    % SW4
    GncdB = SNRdB(Pdp,Pfa,1,4) - SNRdB(Pdp,Pfa,N,4);
    GncN4(p) = (10^(GncdB/10))/N;
    
end

figure
plot(Pd,[GncN0; GncN3; GncN4])
xlabel('Pd'); ylabel('Gnc/N')
grid
title(['N=',num2str(N),', Pfa=',num2str(Pfa)])
hline(1,'b')
legend('SW0','SW3','SW4','Location','Best')
shg


