% SNRdB_as_a_function_of_Pd_and_N
%
% Demo program to show use of SNRdB.m to compute required SNR as a function
% of Pd, Pfa, and N for various target fluctuation models.  In this
% example, Pfa is a fixed value and Pd and N vary.
%
% This program is slow, may take a couple of mintues to run.
%
%  Mark A. Richards
%  May 2010
%

clear all
close all


% set up a grid of pd and N values
pd = [(.1:.05:.75),(.8:.02:.88),(.9:.01:.99)];
n = [1:9,10:2:18,20:5:50];

[PD,N] = meshgrid(pd,n);

% fixed value for Pfa
pfa = 1e-8;
PFA = pfa*ones(size(PD));

% Do it for all the Swerling cases
S0 = SNRdB(PD,PFA,N,0);
S1 = SNRdB(PD,PFA,N,1);
S2 = SNRdB(PD,PFA,N,2);
S3 = SNRdB(PD,PFA,N,3);
S4 = SNRdB(PD,PFA,N,4);
S5 = SNRdB(PD,PFA,N,5);
SA = SNRdB(PD,PFA,N,6);

figure; mesh(pd,n,S0); xlabel('Pd'); ylabel('N'); zlabel('SNR (dB)'); title('S0')
figure; mesh(pd,n,S1); xlabel('Pd'); ylabel('N'); zlabel('SNR (dB)'); title('S1')
figure; mesh(pd,n,S2); xlabel('Pd'); ylabel('N'); zlabel('SNR (dB)'); title('S2')
figure; mesh(pd,n,S3); xlabel('Pd'); ylabel('N'); zlabel('SNR (dB)'); title('S3')
figure; mesh(pd,n,S4); xlabel('Pd'); ylabel('N'); zlabel('SNR (dB)'); title('S4')
figure; mesh(pd,n,S5); xlabel('Pd'); ylabel('N'); zlabel('SNR (dB)'); title('S5 (same as S0)')
figure; mesh(pd,n,SA); xlabel('Pd'); ylabel('N'); zlabel('SNR (dB)'); title('Albersheim')